"""
Auditor has one task: inspection of metadata.json and PPR.xml for required properties
"""
import logging
import os.path
import shutil
from pathlib import Path
from typing import List, Dict
from bs4 import BeautifulSoup
import prettierfier
import json

from casa_envoy.interfaces import AuditorIF
from casa_envoy.schema import AbstractTextFile


def get_fields_for(filename: str) -> list:
    metadata_list = ["fileSetIds", "workflowName", "systemId", "productLocator", "destinationDirectory"]
    ppr_list = ["RootDirectory", "RelativePath", "SdmIdentifier"]

    if ".xml" in filename:
        return ppr_list

    if ".json" in filename:
        return metadata_list


def get_xml_content(filename: str):
    with open(filename) as file:
        return BeautifulSoup(file.read(), "xml")


def get_value_for(filename: str, key: str) -> str:
    if ".xml" in filename:
        ppr_content = get_xml_content(filename)
        return ppr_content.find(key).string

    if ".json" in filename:
        with open(filename) as file:
            metadata = json.loads(file.read())
            return metadata[key]


class AuditFiles(AuditorIF):
    def __init__(self, files: List[str], settings: Dict[str, str]):
        self.files = files
        self.settings = settings
        self.logger = logging.getLogger("casa_envoy")

    def read_file(self, filename: str) -> AbstractTextFile:
        if os.path.isfile(filename):
            with open(filename) as file:
                if ".json" in filename:
                    metadata = json.loads(file.read())
                    return AbstractTextFile(filename, json.dumps(metadata))
                else:
                    if ".xml" in filename:
                        ppr = file.read()
                        return AbstractTextFile(filename, ppr)

    def check_required_fields(self, file: AbstractTextFile, fields: list) -> bool:
        missing = []
        if ".xml" in file.filename:
            ppr_content = BeautifulSoup(file.content, "xml")
            for tag in fields:
                try:
                    ppr_content.find(tag).string
                except:
                    missing.append(tag)

        if ".json" in file.filename:
            for tag in fields:
                metadata = json.loads(file.content)
                if tag not in metadata or len(metadata[tag]) == 0:
                    missing.append(tag)
        if len(missing) > 0:
            print(f"Missing fields: {missing}")
            return False
        else:
            return True

    def correct_for_condor(self, ppr: AbstractTextFile) -> AbstractTextFile:
        shutil.copy(ppr.filename, "./working")
        os.chdir("./working")

        parsed_xml = get_xml_content(ppr.filename)
        parsed_xml.find("RootDirectory").string = self.settings["rootDirectory"]
        parsed_xml.find("RelativePath").string = self.settings["processingDirectory"]

        with open(ppr.filename, "w") as file:
            file.write(prettierfier.prettify_xml(parsed_xml.prettify(), indent=4))

        os.chdir("../")
        return ppr

    def audit(self) -> bool:
        invalid_files = []
        for file in self.files:
            self.logger.info(f"Auditing file {file}...")
            f = self.read_file(file)

            if f.filename == "PPR.xml":
                self.logger.info("Correcting PPR.xml for condor processing...")
                f = self.correct_for_condor(f)

            valid = self.check_required_fields(file=f, fields=get_fields_for(f.filename))
            if not valid:
                invalid_files.append(f.filename)

        if len(invalid_files) != 0:
            self.logger.info(f"INVALID FILE FOUND: {invalid_files}")
            return False
        else:
            return True


class AuditDirectories(AuditorIF):
    def __init__(self, ppr: str):
        self.logger = logging.getLogger("casa_envoy")
        self.rootDirectory = get_value_for(ppr, "RootDirectory")
        self.relative_path = get_value_for(ppr, "RelativePath")
        self.sdmId = get_value_for(ppr, "SdmIdentifier")

    def audit(self) -> bool:
        current = os.getcwd()
        needed = self.rootDirectory + "/" + self.relative_path
        if needed != current:
            self.logger.info("DIRECTORY ERROR: not in correct directory for processing.")
            return False
        else:
            working = Path(current + "/working").is_dir()
            products = Path(current + "/products").is_dir()
            rawdata = Path(current + "/rawdata").is_dir()

            if working and products and rawdata:
                self.logger.info("Checking for rawdata...")
                data = os.listdir(Path(current + "/rawdata/"))
                if len(data) > 0:
                    self.logger.info("Data is available. Proceeding...")
                    return True
                else:
                    self.logger.info("FAILURE: data not found in rawdata/")
                    return False
            else:
                self.logger.info("DIRECTORY ERROR: A directory is missing from the processing root directory.")
