"""
Tests for casa_envoy Auditor
"""
from pathlib import Path
from unittest.mock import mock_open, patch

from casa_envoy.auditor import (
    AuditDirectories,
    AuditFiles,
    get_fields_for,
    get_value_for,
    get_xml_content,
)
from casa_envoy.schema import AbstractTextFile

settings = {
    "useCasa": False,
    "homeForReprocessing": "/home/casa/packages/pipeline/current",
    "rootDirectory": "/tmp/workspaces_tmp/",
    "processingDirectory": "tmpiox5trbp"
}
test_ppr = AbstractTextFile(filename="test/PPR.xml", content=Path("test/PPR.xml").read_text())


def test_get_fields_for():
    fields = ["fileSetIds", "workflowName", "systemId", "productLocator", "destinationDirectory"]
    result = get_fields_for("test/test.json")
    assert result == fields
    fields2 = ["RootDirectory", "RelativePath", "SdmIdentifier"]
    result2 = get_fields_for("test/PPR.xml")
    assert result2 == fields2


def test_get_xml_content():
    content = get_xml_content("test/PPR.xml")
    assert content.find("RelativePath").string == "tmpiox5trbp"
    assert content.find("SdmIdentifier").string == "brain_001.58099.678886747686"


def test_get_value_for():
    value = get_value_for("test/PPR.xml", "RelativePath")
    assert value == "tmpiox5trbp"
    value = get_value_for("test/PPR.xml", "SdmIdentifier")
    assert value == "brain_001.58099.678886747686"

    value = get_value_for("test/test.json", "workflowName")
    assert value == "std_calibration"
    value = get_value_for("test/test.json", "productLocator")
    assert value == "uid://evla/execblock/9cb0964b-ad6b-40ed-bd87-d08c502503e2"


class TestAuditFiles:
    def test_read_file(self):
        file = AuditFiles(["test/test.json", "test/PPR.xml"], settings).read_file("test/test.json")
        assert file.filename == "test/test.json"

        file2 = AuditFiles(["test/test.json", "test/PPR.xml"], settings).read_file("test/PPR.xml")
        assert file2.filename == "test/PPR.xml"

    def test_check_required_fields(self):
        file = AuditFiles(["test/test.json", "test/PPR.xml"], settings).read_file("test/test.json")
        fields = [
            "fileSetIds",
            "workflowName",
            "systemId",
            "productLocator",
            "destinationDirectory",
        ]

        result = AuditFiles(["test/test.json", "test/PPR.xml"], settings).check_required_fields(file, fields)
        assert result is True

        fieldsF = [
            "fileSetIds",
            "workflowName",
            "systemId",
            "productLocator",
            "destinationDirectory",
            "FakeField",
        ]

        resultF = AuditFiles(["test/test.json", "test/PPR.xml"], settings).check_required_fields(
            file, fieldsF
        )
        assert resultF is False

        file2 = AuditFiles(["test/test.json", "test/PPR.xml"], settings).read_file("test/PPR.xml")
        fields2 = ["RootDirectory", "RelativePath", "SdmIdentifier"]

        result2 = AuditFiles(["test/test.json", "test/PPR.xml"], settings).check_required_fields(
            file2, fields2
        )
        assert result2 is True

        fields2F = ["RootDirectory", "RelativePath", "SdmIdentifier", "FakeField"]

        result2F = AuditFiles(["test/test.json", "test/PPR.xml"], settings).check_required_fields(
            file2, fields2F
        )
        assert result2F is False

    @patch("os.chdir")
    @patch("shutil.copy")
    def test_correct_for_condor(self, mock_copy, mock_os):
        ppr = AuditFiles(["test/test.json", "test/PPR.xml"], settings).correct_for_condor(ppr=test_ppr)
        assert ppr.filename == "test/PPR.xml"

    def test_audit(self):
        result = AuditFiles(["test/test.json", "test/PPR.xml"], settings=settings).audit()
        assert result is True


class TestAuditDirectories:
    def test_audit(self):
        result = AuditDirectories("test/PPR.xml").audit()
        assert result is False
